/*
 * main.c
 *
 * Copyright 1998, 1999 Michael Elizabeth Chastain, <mailto:mec@shout.net>.
 * Licensed under the Gnu Public License, version 2.
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "mconfig.h"
#include "parser.tab.h"



/*
 * Main function.
 */

int main( int argc, char * argv [] )
{
    block_type * block_top = NULL;



    /*
     * Parse arguments.
     * Initialize symbol table.
     */

    argument_parse( argc, argv );
    symbol_table_reset( );



    /*
     * Load the config.in file (and everything it sources).
     */

    if ( argument.fs )
    {
	const char * error_string;

	if ( input_push_file( argument.fs, &error_string ) < 0 )
	    error_exit( error_string );

	parser_magic_cookie = COOKIE_CONFIG_LANGUAGE;
	if ( yyparse( ) != 0 || parser_error_count > 0 )
	{
	    if ( argument.mode != mode_syntax )
	    {
		char * buf = grab_memory( strlen(argument.fs) + 32 );
		sprintf( buf, "%s: failed to parse", argument.fs );
		error_exit( buf );
	    }
	}

	if ( parser_warning_count > 0 )
	    sleep( 2 );

	block_top = parser_block_top;
    }



    /*
     * Load old values from $file-config-in.
     *
     * FIXME: this code can be made into a "load" function
     *   (that is why it has a redundant symbol table reset).
     */

    if ( argument.fci != NULL )
    {
	const char * error_string;
	def_line_type * def_line;

	if ( input_push_file( argument.fci, &error_string ) < 0 )
	    error_exit( error_string );

	parser_magic_cookie = COOKIE_DEF_CONFIG;
	if ( yyparse( ) != 0 || parser_error_count > 0 )
	{
	    if ( argument.mode != mode_syntax )
	    {
		char * buf = grab_memory( strlen(argument.fci) + 32 );
		sprintf( buf, "%s: failed to parse", argument.fci );
		error_exit( buf );
	    }
	}

	symbol_table_reset( );
	for ( def_line  = parser_def_line_list->first;
	      def_line != NULL;
	      def_line  = def_line->next )
	{
	    def_line->symbol->value  = def_line->value;
	    def_line->symbol->origin = origin_dotconfig;
	}
    }



    /*
     * Run the major mode.
     */

    switch ( argument.mode )
    {
    default:           error_enum_bogus ( ); break;
    case mode_menu:    do_mode_menu     ( block_top ); break;
    case mode_x_gtk:   do_mode_x_gtk    ( block_top ); break;
    case mode_line:    do_mode_line     ( block_top ); break;
    case mode_old:     do_mode_old      ( block_top ); break;
    case mode_syntax:  do_mode_syntax   ( block_top ); break;
    case mode_random:  do_mode_random   ( block_top ); break;
    case mode_mutate:  do_mode_mutate   ( block_top ); break;
    case mode_minimum: do_mode_minimum  ( block_top ); break;
    case mode_maximum: do_mode_maximum  ( block_top ); break;
    }

    return 0;
}
